package br.com.tecpoint.api.impl;

import br.com.tecpoint.api.*;
import br.com.tecpoint.api.win32.W32API.HWND;
import com.sun.jna.Function;
import com.sun.jna.NativeLibrary;
import com.sun.jna.Pointer;

/**
 * Tecpoint VisionAPI version for Panini readers.<br>
 * IMPORTANT: dependency 'jna.jar' - Java Native Access(JNA), download http://jna.java.net/<br>
 * <br>1.01 version changed<br>
 * getDocumentType(int DocID, boolean bwarning) to getDocumentType(int DocID);;
 *
 * @see VAPIInterface  
 * @version 1.01
 * @since Maio/2011
 * @author Cirilo Veloso - http://wwww.cjv.com.br
 * 
 */
public class VisionAPIImpl implements VAPIInterface {
    public static int deviceID = 0;
    private String java_debug_string="***JAVA VISION API DEBUG**** ->  ";
    private boolean debugMode = false;
    private static final String API = "VisionAPI";
    private static final String HLP = "MvxApiHlp";
    private NativeLibrary dll = NativeLibrary.getInstance(API);
    private NativeLibrary dllHelper = NativeLibrary.getInstance(HLP);
    private boolean boolRet;
    private int intRet;
    private int errorCod;
    private String errorString = "";
    private Function proxy;

    public VisionAPIImpl() {
        
    }


    /**
     * This param enable debug mode, System.out.println()
     * @param debugMode
     */
    public VisionAPIImpl(boolean debugMode) {
        this.debugMode = debugMode;
    }

    /**
     *
     * @return boolean
     */
    public boolean isDebugMode() {
        return debugMode;
    }

    /**
     *
     * @param debugMode
     */
    public void setDebugMode(boolean debugMode) {
        this.debugMode = debugMode;
    }

    /**
     * This function is used to obtain the VisionAPI interface release, which is returned as a NULL terminated string.
     * @see setDeviceEngine()
     * @return String
     */
    public String getVersion() {
    	if(debugMode){System.out.println(java_debug_string+"vApiGetVersion()");}
        proxy = dll.getFunction(Utils.getMethods().get("VApiGetVersion"));
        byte result[] = new byte[256];
        int maxLen = 250;
        String strVer = "";
        Object[] par = new Object[]{result, maxLen};
        intRet = (Integer) proxy.invoke(Integer.class, par);
        if (intRet == 0) {
            strVer = Utils.byteToString(result);
        }        
        return strVer;
    }

    /**
     * This function is used to load an underlying device engine. This function is intended for future
        use, in order a different device engine to drive a specific Panini device. Now the unique
        supported function device is the MyVisionX / Vision|X. If this function is not called the
        VisionAPI automatically load the Vision|X device layer.<br>
        It must be called as the first function or after a “Shutdown” call, in other words when no
        devices are connected.<br>
     * @param engineSelector set the engine to be loaded. The valid values for<br>
        this parameter are defined in the header file,<br> DEVICE_MYVISIONX  = 0;<br>
	DEVICE_VISIONX    = 1;<br>
	DEVICE_VISIONS    = 2;<br>
	DEVICE_IDEAL      = 3;<br>
     * @return int
     * @see getVersion()
     */
    public int setDeviceEngine(int engineSelector) {
    	if(debugMode){System.out.println(java_debug_string+"vApiSetDeviceEngine");}
        proxy = dll.getFunction(Utils.getMethods().get("VApiSetDeviceEngine"));
        Object[] par = new Object[]{engineSelector};
        intRet = (Integer) proxy.invoke(Integer.class, par);
        return intRet;
    }

    /**
     * This function gets the last occurred error. It has to be called after a function call failure to
       know the reason of the problem. Use this function with the other device engine layer to
       understand the reason of a function call failure.
     * @return int
     * @see getErrorString()
     */
    public int getError() {
    	if(debugMode){System.out.println(java_debug_string+"vApiGetError");}
        proxy = dll.getFunction(Utils.getMethods().get("VApiGetError"));
        errorCod = (Integer) proxy.invoke(Integer.class, new Object[]{});
        return errorCod;
    }

    /**
     * This function gets an error code description. After a call to the VApiGetError the programmer
       can use this function to obtain a description of the error.
     * @return String
     * @see getError()
     */
    public String getErrorString() {
    	if(debugMode){System.out.println(java_debug_string+"vApiGetErrorString");}
        proxy = dll.getFunction(Utils.getMethods().get("VApiGetErrorString"));
        byte result[] = new byte[256];
        int maxlen = 250;
        errorCod = getError();
        if (errorCod != 0) {
            Object[] par = new Object[]{errorCod, result, maxlen};
            intRet = (Integer) proxy.invoke(Integer.class, par);
            if (intRet == 0) {
                errorString = Utils.byteToString(result);
            }
        }
        return errorString;
    }

    /**
     * This function is used to obtain the API release, which is returned as a NULL terminated string.<br>
            Valid in State: All<br>
            State Transition: None<br>
     * @return String
     * @see getDriverRelease()
     */
    public String getApiRelease() {
    	if(debugMode){System.out.println(java_debug_string+"getApiRelease");}
        proxy = dll.getFunction(Utils.getMethods().get("GetApiRelease"));
        byte result[] = new byte[256];
        byte maxLen = (byte) 250;
        String strVer = "";
        Object[] par = new Object[]{result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strVer = Utils.byteToString(result);
        }
        return strVer;
    }

    /**
     * This function is used to obtain the driver release, which is returned as a NULL terminated string.<br>Valid in State: All
            State Transition: None<br>
     * @return String
     * @see getApiRelease()
     */
    public String getDriverRelease() {
    	if(debugMode){System.out.println(java_debug_string+"getDriverRelease");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDriverRelease"));
        byte result[] = new byte[256];
        byte maxLen = (byte) 250;
        String strVer = "";
        Object[] par = new Object[]{result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strVer = Utils.byteToString(result);
        }
        return strVer;
    }

    /**
     * This function is used to obtain the Firmware version, which is returned as a NULL terminated
        string. It can be called only after the device is connected, because it is retrieved directly from the device.<br>
        Valid in State: DeviceChangeParameters, DeviceOnLine, DeviceOffLine<br>
        State Transition: None<br>
     * @return String
     * @see getSerialNumber()
     */
    public String getFwVersion() {
    	if(debugMode){System.out.println(java_debug_string+"getFwVersion");}
        proxy = dll.getFunction(Utils.getMethods().get("GetFwVersion"));
        byte result[] = new byte[256];
        byte maxLen = (byte) 250;
        String strVer = "";
        Object[] par = new Object[]{deviceID, result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strVer = Utils.byteToString(result);
        }
        return strVer;
    }

    /**
     * This function is used to obtain the Serial number of the connected reader, which is returned as
            a NULL terminated string. It can be called only after the device is connected, because it is
            retrieved directly from the device.<br>
            Valid in State: DeviceChangeParameters, DeviceOnLine, DeviceOffLine<br>
            State Transition: None<br>
     * @return String
     * @see getFwVersion()
     */
    public String getSerialNumber() {
    	if(debugMode){System.out.println(java_debug_string+"getSerialNumber");}
        proxy = dll.getFunction(Utils.getMethods().get("GetSerialNumber"));
        byte result[] = new byte[15];
        byte maxLen = (byte) 15;
        String strRet = "";
        Object[] par = new Object[]{deviceID, result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strRet = Utils.byteToString(result);
        }
        return strRet;
    }

    /**
     * This function is used to obtain the IDCard encrypt code.<br>
            Valid in State: DeviceChangeParameters<br>
            State Transition: None<br>
     * @return String
     * @see getIDCardDescription()
     */
    public String readCryptedIDCard() {
    	if(debugMode){System.out.println(java_debug_string+"readCryptedIDCard");}
        proxy = dll.getFunction(Utils.getMethods().get("ReadCryptedIDCard"));
        byte result[] = new byte[256];
        byte pLen[] = new byte[512];
        String strRet = "";
        Object[] par = new Object[]{deviceID, result, pLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strRet = Utils.byteToString(result);
        }
        return strRet;
    }

    /*
    public String getIDCardDescription(int[] pLen, Devices[] devices) {
    	if(debugMode){System.out.println(java_debug_string+"getIDCardDescription");}
        proxy = dll.getFunction(Utils.getMethods().get("GetIDCardDescription"));
        char result[] = new char[256];    
        String strRet = "";
        Object[] par = new Object[]{deviceID, result, pLen, devices};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {      
            strRet = Native.toString(result);
        }
        return strRet;
    }
     */

    /**
     * This function is used to obtain the description of the device from the IDCard, which stores all
       information about rights and licenses. The buffer for the description is a pointer address
       supplied by the application and will be internally allocated, so remember to free it with
       VirtualFree() when it is no longer needed. The DEVICES structure will contain the features
       of the device. The DEVICES structure members are the following:<br>
       char SerialNumber[11] : Serial number 53xxxxx<br>
       BYTE FeederLimit : Valid values are 0(Unlimited), or 30<br>
       BYTE MaxDPM : Document per minute<br>
       BOOL MicrE13B : TRUE means device available<br>
       BOOL MicrCMC7 : TRUE means device available<br>
       BOOL MicrAUTO : TRUE means device available<br>
       BOOL Inkjet : TRUE means device available<br>
       BOOL InkjetMultiline : TRUE means device available<br>
       BOOL InkjetGraphic : TRUE means device available<br>
       BYTE InkjetLines : Valid values are 0 (Unlimited), 1, 2 and 3 lines.<br>
       BOOL ImageFront : TRUE means device available<br>
       BOOL ImageFrontClr : TRUE means device available<br>
       BOOL ImageFrontDropOut : TRUE means device available<br>
       BOOL ImageRear : TRUE means device available<br>
       BOOL ImageRearClr : TRUE means device available<br>
       BOOL ImageRearDropOut : TRUE means device available<br>
       BOOL OcrAB : TRUE means device available<br>
       BOOL OcrMicr : TRUE means device available<br>
       BOOL OcrBarcode1D : TRUE means device available<br>
       BOOL OcrBarcode2D : TRUE means device available<br>
       Valid in State: DeviceChangeParameters<br>
       State Transition: None<br>
     * @param pLen   Receive the IDCard description’s length.
     * @param devices  the address of the pointer where to allocate and store
                        the device description received from the IDCard.
     * @return String
     * @see readCryptedIDCard(()
     */
    public String getIDCardDescription(int[] pLen, Devices[] devices) {
    	if(debugMode){System.out.println(java_debug_string+"getIDCardDescription");}
        //int GetIDCardDescHelp(unsigned long,char *,long,struct _tagDevices *)
        proxy = dllHelper.getFunction(Utils.getMethods().get("GetIDCardDescHelp"));
        byte result[] = new byte[pLen[0]];
        String strRet = "";
        Object[] par = new Object[]{deviceID, result, pLen[0], devices};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strRet = Utils.byteToString(result);
        }
        return strRet;
    }
    
    /**
     * Use this function to upgrade a device using an IDCard Upgrade buffer. If the missing of a
            prerequisite is detected a DEVICE_ERR_NO_PREREQUISITE_FOR_UPGRADE error is
            retrieved.<br>
            Valid in State : DeviceChangeParameters<br>
            State Transition : None<br>
     * @param len  - it’s the length of the upgrade buffer (256 bytes).
               I-Deal requires 1024 bytes.
     * @return boolean
     * @see getDeviceFeature
     */
    public boolean upGradeIDCard(int len) {
    	if(debugMode){System.out.println(java_debug_string+"upGradeIDCard");}
        proxy = dll.getFunction(Utils.getMethods().get("UpgradeIDCard"));
        byte[] pIDCardUpgrade = new byte[len];
        Object[] par = new Object[]{deviceID, pIDCardUpgrade, len};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function is used to obtain information, capabilities and licenses of the device. Most of
            the information returned by this function is equal to the ones returned by the
            GetIDCardDescription function.<br>
            The information are requested passing a feature ID number (FeatureID) to the function
            that returns the corresponding data in the user allocated buffer (lpReturnedBuffer). The
            buffer must be big enough to contain the data (BufferSize).<br>
            The data are defined as DWORD (32 bits) or char string. The features ID are defined in the
            header file.<br> Valid in State: DeviceChangeParameters, DeviceOnLine<br>
            State Transition: None<br>
     * @param deviceFeature It’s the requested Feature ID number
     * @return Object ( byte[] or int[] )
     * @see getDeviceFeature()
     */
    public Object getDeviceFeature(int deviceFeature) {
    	if(debugMode){System.out.println(java_debug_string+"getDeviceFeature");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDeviceFeature"));
        Object[] par;
        Object result;
        switch (deviceFeature) {
            case VAPIInterface.DEVICE_FEATURE_SN: {
                result = (byte[]) new byte[15];
                par = new Object[]{deviceID, deviceFeature, result, 15};
                break;
            }
            case VAPIInterface.DEVICE_FEATURE_SN_IDCARD: {
                result = (byte[]) new byte[15];
                par = new Object[]{deviceID, deviceFeature, result, 15};
                break;
            }
            case VAPIInterface.DEVICE_FEATURE_SN_EXTERNAL_IDCARD: {
                result = (byte[]) new byte[250];
                par = new Object[]{deviceID, deviceFeature, result, 250};
                break;
            }
            default: {
                result = (int[]) new int[4];
                par = new Object[]{deviceID, deviceFeature, result, 4};
                break;
            }
        }
        
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (result instanceof byte[]) {
            return Utils.byteToString((byte[]) result);
        } else {
            return result;
        }

    }

    /**
     * Use this function to retrieve information if a USB error occurs. pdwError will be the USB
        error code (see Error Handling Chapter for more information).<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return int
     * @see getUsbErrorString()
     */
    public int getUsbError() {
    	if(debugMode){System.out.println(java_debug_string+"getUsbError");}
        proxy = dll.getFunction(Utils.getMethods().get("GetUsbError"));
        int result[] = new int[1];
        int intError = 0;
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            intError = result[0];
        }
        return intError;
    }

    /**
     * Use this function to retrieve information if a USB error occurs. The pointer pcErrorString will
        be the USB error description string.<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return String
     * @see getUsbError()
     */
    public String getUsbErrorString() {
    	if(debugMode){System.out.println(java_debug_string+"getUsbErrorString");}
        proxy = dll.getFunction(Utils.getMethods().get("GetUsbErrorString"));
        byte result[] = new byte[200];
        byte maxLen = (byte) 200;
        String strError = "";
        Object[] par = new Object[]{deviceID, result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strError = Utils.byteToString(result);
        }
        return strError;
    }

   /**
    * Use this function to retrieve information if a Device error occurs. pdwError will be the Device
        error code (see Error Handling Chapter for more information).<br>
        Valid in State : All<br>
        State Transition : None<br>
    * @return int
    * @see getDeviceErrorString()
    */
    public int getDeviceError() {
    	if(debugMode){System.out.println(java_debug_string+"getDeviceError");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDeviceError"));
        int result[] = new int[1];
        int intError = 0;
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            intError = result[0];
        }
        return intError;
    }

   /**
    * Use this function to retrieve information if a Device error occurs. pcErrorString will be the
        Device error description string.<br>
        Valid in State : All<br>
        State Transition : None<br>
    * @return String
    * @see getDeviceError()
    */
    public String getDeviceErrorString() {
    	if(debugMode){System.out.println(java_debug_string+"getDeviceErrorString");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDeviceErrorString"));
        byte result[] = new byte[200];
        byte maxLen = (byte) 200;
        String strError = "";
        Object[] par = new Object[]{result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strError = Utils.byteToString(result);
        }
        return strError;
    }

    /**
     * Use this function to retrieve information if an API error occurs. pdwError will be the API
        error code (see Error Handling Chapter for more information).<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return int
     * @see getApiErrorString()
     */
    public int getApiError() {
    	if(debugMode){System.out.println(java_debug_string+"getApiError");}
        proxy = dll.getFunction(Utils.getMethods().get("GetApiError"));
        int result[] = new int[1];
        int intCod = 0;
        Object[] par = new Object[]{result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            intCod = result[0];
        }
        return intCod;
    }

    /**
     * Use this function to retrieve information if an API error occurs. pcErrorString will be the API
        error description string.<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return String
     * @see getApiError()
     */
    public String getApiErrorString() {
    	if(debugMode){System.out.println(java_debug_string+"getApiErrorString");}
        proxy = dll.getFunction(Utils.getMethods().get("GetApiErrorString"));
        byte result[] = new byte[200];
        byte maxLen = (byte) 200;
        String strError = "";
        Object[] par = new Object[]{result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strError = Utils.byteToString(result);
        }
        return strError;
    }


    /**
     * Use this function to retrieve information directly from the firmware, it is to be used only when
        more information about a problem is needed. pdwError will be the Sorter error code (see
        Error Handling Chapter for more information).<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return int
     * @see getSorterErrorString()
     */
    public int getSorterError() {
    	if(debugMode){System.out.println(java_debug_string+"getSorterError");}
        proxy = dll.getFunction(Utils.getMethods().get("GetSorterError"));
        int result[] = new int[1];
        int intError = 0;
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            intError = result[0];
        }
        return intError;
    }

    /**
     * Use this function to retrieve information directly from the firmware, it is to be used only when
        more information about a problem is needed. pcErrorString will be the Sorter error
        description string.<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return String
     * @see getSorterError()
     */
    public String getSorterErrorString() {
    	if(debugMode){System.out.println(java_debug_string+"getSorterErrorString");}
        proxy = dll.getFunction(Utils.getMethods().get("GetSorterErrorString"));
        byte result[] = new byte[200];
        int maxLen = 200;
        String strResult = "";
        Object[] par = new Object[]{deviceID, result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strResult = Utils.byteToString(result);
        }
        return strResult;
    }

    /**
     * Use this function to retrieve the current status code of the device.<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return int
     * @see getDeviceStateString()
     */
    public int getDeviceState() {
    	if(debugMode){System.out.println(java_debug_string+"getDeviceState");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDeviceState"));
        int result[] = new int[1];
        int intState = 0;
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            intState = result[0];
        }
        return intState;
    }

    /**
     * Use this function to retrieve the Device State string. pcDeviceStateString will be the Device
        State string.<br>
        Valid in State : All<br>
        State Transition : None<br>
     * @return String
     * @see getDeviceState()
     */
    public String getDeviceStateString() {
    	if(debugMode){System.out.println(java_debug_string+"getDeviceStateString");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDeviceStateString"));
        byte result[] = new byte[200];
        int maxLen = 200;
        String strState = "";
        Object[] par = new Object[]{deviceID, result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strState = Utils.byteToString(result);
        }
        return strState;
    }


    /**
     * This function call opens a communication channel between the device and the application, if
        the Startup is successful the Device Identification Number is returned.<br>
        Valid in State : DeviceShutDown<br>
        State Transition : To DeviceStartingUp and then to DeviceChangeParameters<br>
     * @param HWND Handle, the handle to the application’s messages destination window
     * @return int
     * @see shutDown()
     */
    public int startUp(HWND h) {
    	int handle=Utils.hwndToInt(h);
    	if(debugMode){System.out.println(java_debug_string+"startUp");}
        proxy = dll.getFunction(Utils.getMethods().get("StartUp"));
        Object[] par = new Object[]{handle, VAPIInterface.WM_PANINI};
        deviceID = (Integer) proxy.invoke(Integer.class, par);
        return deviceID;
    }

    /**
     * This function closes the communication channel between the sorter and the pc<br>
        Valid in State : DeviceChangeParameters, DeviceOnLine, DeviceOffLine, DeviceStandBy<br>
        State Transition : to DeviceShutDown<br>
     * @return boolean
     * @see startUp()
     */
    public boolean shutDown() {
    	if(debugMode){System.out.println(java_debug_string+"shutDown");}
        proxy = dll.getFunction(Utils.getMethods().get("ShutDown"));
        Object[] par = new Object[]{deviceID};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function forces the device in the DeviceChangeParameters state.<br>
        Valid in State : DeviceOnLine, DeviceOffLine<br>
        State Transition : to DeviceChangeParameters<br>
     * @return boolean
     * @see onLine()
     */
    public boolean changeParameters() {
    	if(debugMode){System.out.println(java_debug_string+"changeParameters");}
        proxy = dll.getFunction(Utils.getMethods().get("ChangeParameters"));
        Object[] par = new Object[]{deviceID};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function forces the device into the OnLine state which is the operative state.<br>
        Valid in State : DeviceChangeParameters, DeviceOffLine<br>
        State Transition : to DeviceOnLine<br>
     * @return boolean
     * @see changeParameters()
     */
    public boolean onLine() {
    	if(debugMode){System.out.println(java_debug_string+"onLine");}
        proxy = dll.getFunction(Utils.getMethods().get("OnLine"));
        Object[] par = new Object[]{deviceID};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * Set a sorter parameter (see the chapter on FW Parameter Description).<br>
        Valid in State : DeviceChangeParameters, DeviceOffLine, DeviceOnLine<br>
        State Transition : none<br>
     * @param paramId the identification of the parameter that will be changed.
     * @param paramValue the value that will be set for the selected unParamId.
     * @return boolean
     * @see getSorterParameter()
     */
    public boolean setSorterParameter(short paramId, short paramValue) {
    	if(debugMode){System.out.println(java_debug_string+"setSorterParameter");}
        proxy = dll.getFunction(Utils.getMethods().get("SetSorterParameter"));
        Object[] par = new Object[]{deviceID, paramId, paramValue};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * Retrieve from the FW a sorter parameter (see the chapter on FW Parameter Description).
        AParameter is a structure defined in this way:<br>
·       sDescription : String describing the Parameter<br>
·       sUnit : String describing the usable unit for the parameter<br>
·       usValue : Actual value of the parameter<br>
·       usDefault : Default value of the parameter<br>
·       usMin : Minimum value available<br>
·       usMax : Maximum value available<br>
        Valid in State : DeviceChangeParameters, DeviceOffLine, DeviceOnLine<br>
        State Transition : none<br>
     * @param paramId the identification of the parameter for which the value will be asked.
     * @return Parameter
     * @see setSorterParameter()
     */
    public Parameter getSorterParameter(short paramId) {
    	if(debugMode){System.out.println(java_debug_string+"getSorterParameter");}
        proxy = dll.getFunction(Utils.getMethods().get("GetSorterParameter"));
        Parameter result = new Parameter();
        Object[] par = new Object[]{deviceID, paramId, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return result;
    }

    /**
     * Set the Device parameter structure containing the processing options setting in the API (see
        the chapter about the DeviceParameters structure).<br>
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @param deviceParam - the structure containing all the processing settings.
     * @return boolean
     * @see getDeviceParameters()
     */
    public boolean setDeviceParameters(DeviceParameters deviceParam) {
        return setDeviceParametersHelp2(deviceParam);
    }

    /**
     *  Set the Device parameter structure containing the processing options setting in the API (see
        the chapter about the DeviceParameters structure).<br>
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @return boolean
     * @see getDeviceParameters()
     */
    public boolean setDeviceParametersDefault(){    	
    	if(debugMode){System.out.println(java_debug_string+"setDeviceParametersDefault");}
        proxy = dllHelper.getFunction(Utils.getMethods().get("SetDeviceParametersHelpDefault"));
        Object[] par = new Object[]{deviceID};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if(!boolRet ){
            errorString = getApiErrorString();
        }
        return boolRet;
    }

   /**
    * Set the Device parameter structure containing the processing options setting in the API (see
        the chapter about the DeviceParameters structure).<br>
        Valid in State : DeviceChangeParameters
        State Transition : none<br>
    * @param devp  the structure containing all the processing settings.
    * @return boolean
    * @see getDeviceParameters()
    */
   public boolean setDeviceParametersHelp2(DeviceParameters devp){
    	if(debugMode){System.out.println(java_debug_string+"setDeviceParametersHelp2");}
        proxy = dllHelper.getFunction(Utils.getMethods().get("SetDeviceParametersHelp2"));
        if(debugMode){System.out.println(java_debug_string+"setDeviceParametersHelp2---- devp"+devp.ImagePropertiesFront1.Format);}
        Object[] par = new Object[]{deviceID,devp.bMICREnable,devp.bMICRSaveSamples,devp.nMICRFont,devp.nMICRSpaces,devp.nReserved,devp.nFeedingMode,devp.bOneDoc,devp.bPrintEnable,devp.bReserved,devp.cRejectSymbol,
        devp.ImagePropertiesFront1.Format,devp.ImagePropertiesFront1.ColorDepth,devp.ImagePropertiesFront1.Paging,devp.ImagePropertiesFront1.Resolution,devp.ImagePropertiesFront1.Threshold,
        devp.ImagePropertiesFront2.Format,devp.ImagePropertiesFront2.ColorDepth,devp.ImagePropertiesFront2.Paging,devp.ImagePropertiesFront2.Resolution,devp.ImagePropertiesFront2.Threshold,
        devp.ImagePropertiesRear1.Format,devp.ImagePropertiesRear1.ColorDepth,devp.ImagePropertiesRear1.Paging,devp.ImagePropertiesRear1.Resolution,devp.ImagePropertiesRear1.Threshold,
        devp.ImagePropertiesRear2.Format,devp.ImagePropertiesRear2.ColorDepth,devp.ImagePropertiesRear2.Paging,devp.ImagePropertiesRear2.Resolution,devp.ImagePropertiesRear2.Threshold,
        devp.SnippetProperties0.Enable,devp.SnippetProperties0.Front,devp.SnippetProperties0.Properties.Color,devp.SnippetProperties0.Properties.Height,devp.SnippetProperties0.Properties.Width,devp.SnippetProperties0.Properties.Millimeters,devp.SnippetProperties0.Properties.Orientation,devp.SnippetProperties0.Properties.Xposition,devp.SnippetProperties0.Properties.Yposition,devp.SnippetProperties0.Properties.Compression,
        devp.SnippetProperties1.Enable,devp.SnippetProperties1.Front,devp.SnippetProperties1.Properties.Color,devp.SnippetProperties1.Properties.Height,devp.SnippetProperties1.Properties.Width,devp.SnippetProperties1.Properties.Millimeters,devp.SnippetProperties1.Properties.Orientation,devp.SnippetProperties1.Properties.Xposition,devp.SnippetProperties1.Properties.Yposition,devp.SnippetProperties1.Properties.Compression,
        devp.SnippetProperties2.Enable,devp.SnippetProperties2.Front,devp.SnippetProperties2.Properties.Color,devp.SnippetProperties2.Properties.Height,devp.SnippetProperties2.Properties.Width,devp.SnippetProperties2.Properties.Millimeters,devp.SnippetProperties2.Properties.Orientation,devp.SnippetProperties2.Properties.Xposition,devp.SnippetProperties2.Properties.Yposition,devp.SnippetProperties2.Properties.Compression,
        devp.SnippetProperties3.Enable,devp.SnippetProperties3.Front,devp.SnippetProperties3.Properties.Color,devp.SnippetProperties3.Properties.Height,devp.SnippetProperties3.Properties.Width,devp.SnippetProperties3.Properties.Millimeters,devp.SnippetProperties3.Properties.Orientation,devp.SnippetProperties3.Properties.Xposition,devp.SnippetProperties3.Properties.Yposition,devp.SnippetProperties3.Properties.Compression,
        devp.SnippetProperties4.Enable,devp.SnippetProperties4.Front,devp.SnippetProperties4.Properties.Color,devp.SnippetProperties4.Properties.Height,devp.SnippetProperties4.Properties.Width,devp.SnippetProperties4.Properties.Millimeters,devp.SnippetProperties4.Properties.Orientation,devp.SnippetProperties4.Properties.Xposition,devp.SnippetProperties4.Properties.Yposition,devp.SnippetProperties4.Properties.Compression,
        devp.SnippetProperties5.Enable,devp.SnippetProperties5.Front,devp.SnippetProperties5.Properties.Color,devp.SnippetProperties5.Properties.Height,devp.SnippetProperties5.Properties.Width,devp.SnippetProperties5.Properties.Millimeters,devp.SnippetProperties5.Properties.Orientation,devp.SnippetProperties5.Properties.Xposition,devp.SnippetProperties5.Properties.Yposition,devp.SnippetProperties5.Properties.Compression,
        devp.SnippetProperties6.Enable,devp.SnippetProperties6.Front,devp.SnippetProperties6.Properties.Color,devp.SnippetProperties6.Properties.Height,devp.SnippetProperties6.Properties.Width,devp.SnippetProperties6.Properties.Millimeters,devp.SnippetProperties6.Properties.Orientation,devp.SnippetProperties6.Properties.Xposition,devp.SnippetProperties6.Properties.Yposition,devp.SnippetProperties6.Properties.Compression,
        devp.SnippetProperties7.Enable,devp.SnippetProperties7.Front,devp.SnippetProperties7.Properties.Color,devp.SnippetProperties7.Properties.Height,devp.SnippetProperties7.Properties.Width,devp.SnippetProperties7.Properties.Millimeters,devp.SnippetProperties7.Properties.Orientation,devp.SnippetProperties7.Properties.Xposition,devp.SnippetProperties7.Properties.Yposition,devp.SnippetProperties7.Properties.Compression,
        devp.SnippetProperties8.Enable,devp.SnippetProperties8.Front,devp.SnippetProperties8.Properties.Color,devp.SnippetProperties8.Properties.Height,devp.SnippetProperties8.Properties.Width,devp.SnippetProperties8.Properties.Millimeters,devp.SnippetProperties8.Properties.Orientation,devp.SnippetProperties8.Properties.Xposition,devp.SnippetProperties8.Properties.Yposition,devp.SnippetProperties8.Properties.Compression,
        devp.SnippetProperties9.Enable,devp.SnippetProperties9.Front,devp.SnippetProperties9.Properties.Color,devp.SnippetProperties9.Properties.Height,devp.SnippetProperties9.Properties.Width,devp.SnippetProperties9.Properties.Millimeters,devp.SnippetProperties9.Properties.Orientation,devp.SnippetProperties9.Properties.Xposition,devp.SnippetProperties9.Properties.Yposition,devp.SnippetProperties9.Properties.Compression};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if(!boolRet ){
            errorString = getApiErrorString();
        }
        return boolRet;
    }

   /**
    * Retrieve the actual Device parameter structure (see the chapter about the DeviceParameters structure).<br>
       Valid in State : DeviceChangeParameters<br>
       State Transition : none<br>
    * @return DeviceParameters
    * @see setDeviceParameters()
    */
    public DeviceParameters getDeviceParameters() {
    	if(debugMode){System.out.println(java_debug_string+"GetDeviceParameters");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDeviceParameters"));
        DeviceParameters result[] = new DeviceParameters[100];
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {          
            return result[0];
        } else {
            return new DeviceParameters();
        }
    }

    /**
     * Set the parameters for the image adjustment. These parameters affect the result of the image
        acquisition. Valid values for Contrast and Brightness are from -100 to 100. The API default is
        zero for both, which means there is no correction applied.
        NOTICE: This function is for special image requirements. It should not be used in “normal” contest.<br>
        Valid in State : DeviceChangeParameters, DeviceOnLine<br>
        State Transition : none<br>
     * @param constrast  image contrast adjustment
     * @param brightness  image brightness adjustment
     * @param front - if TRUE the adjustment is related to the front image, FALSE to the rear image.
     * @return boolean
     * @see setMaxDpm()
     */
    public boolean setImageAdjustment(short constrast, short brightness, boolean front) {
    	if(debugMode){System.out.println(java_debug_string+"SetImageAdjustment");}
        proxy = dll.getFunction(Utils.getMethods().get("SetImageAdjustment"));
        Object[] par = new Object[]{deviceID, constrast, brightness, front};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function is intended for demo purposes. Set the DPM value (according to the max
        available for that device, if there is an attempt to increase this number above the max possible
        an error is returned, the max available DPM is stored in the IDCard). Use GetMaxDpm
        function to retrieve the actual DPM setting. The available DPM values are the following:<br>
       | DPM     |Doc length independent  | Doc length  dependent(Referred to 6” (152mm) doc length)|<br>
       | 30      |         X              |                                                         |<br>
       | 60      |         X              |                                                         |<br>
       | 90      |         X              |                                                         |<br>
       | 50      |                        |             X                                           |<br>
       | 75      |                        |             X                                           |<br>
       | 100     |                        |             X                                           |<br>
       The DPM dependent on doc length means that the documents throughput is declared using the
       reference document length. The reference document length is 6” (152 mm). This means that a
       shorter document results in a higher DPM. A longer one results in a lower DPM. The
       MyVisionX supports 30, 60 and 90. Vision|X support 50, 75 and 100 DPM. The VisionS
       supports each single DPM value up to 200 (maximum).<br>
       Valid in State : DeviceChangeParameters, DeviceOffLine, DeviceOnLine<br>
       State Transition : none<br>
     * @param dpm the number of Dpm to be set.
     * @return boolean
     * @see getMaxDPM()
     */
    public boolean setMaxDPM(int dpm) {
    	if(debugMode){System.out.println(java_debug_string+"SetMaxDPM");}
        proxy = dll.getFunction(Utils.getMethods().get("SetMaxDPM"));
        Object[] par = new Object[]{deviceID, dpm};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * Get the actual Dpm value. Use SetMaxDpm function to change the actual Dpm setting.<br>
        Valid in State : DeviceChangeParameters, DeviceOffLine, DeviceOnLine<br>
        State Transition : none<br>
     * @return int
     * @see setMaxDPM()
     */
    public int getMaxDPM() {
    	if(debugMode){System.out.println(java_debug_string+"GetMaxDPM");}
        proxy = dll.getFunction(Utils.getMethods().get("GetMaxDPM"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        } else {
            return 0;
        }
    }

    /**
     * This function sets the enabled lines for the AGP printer (according to the max available for
        that device, if there is an attempt to increase this number above the max possible an error is
        returned, the max available lines is stored in the IDCard). Use GetIDCardDesciption function
        to retrieve the actual enabled lines. Valid values for AGP lines are defined in the header file.
        This function is intended for demo purposes.<br>
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @param lines the number of lines to be set.
     * @return boolean
     * @see getAGPLines()
     */
    public boolean setAGPLines(byte lines) {
    	if(debugMode){System.out.println(java_debug_string+"SetAGPLines");}
        proxy = dll.getFunction(Utils.getMethods().get("SetAGPLines"));
        Object[] par = new Object[]{deviceID, lines};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function sets the feeder limitation (according to the max available for that device, if there
        is an attempt to increase this number above the max possible an error is returned, the max
        available lines is stored in the IDCard). Use GetIDCardDesciption function to retrieve the
        actual feeder limit. Valid values for Limit are defined in the header file.
        This function is intended for demo purposes.<br>
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @param limit the feeder limit
     * @return boolean
     * @see setFeederLimit()
     */
    public boolean setFeederLimit(byte limit) {
    	if(debugMode){System.out.println(java_debug_string+"SetFeederLimit");}
        proxy = dll.getFunction(Utils.getMethods().get("SetFeederLimit"));
        Object[] par = new Object[]{deviceID, limit};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * When the feeder is in Hand-Drop mode and the device is in Feeding state, when a document
       is detected in the feeder, the device waits a certain delay and then feeds the document in the
       track. This function can modify this delay.
       The Delay range is from 100 to 60000 ms. The API default is 100 ms.<br>
       Valid in State : DeviceChangeParameters<br>
       State Transition : none<br>
     * @param dly - The delay expressed in ms
     * @return boolean
     * @see getAvailablePockets()
     */
    public boolean setHandDropDly(int dly) {
    	if(debugMode){System.out.println(java_debug_string+"setHandDropDly");}
        proxy = dll.getFunction(Utils.getMethods().get("SetHandDropDly"));
        Object[] par = new Object[]{deviceID, dly};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * Get the pockets installed and enabled on the device.<br>
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @return byte
     * @see setMaxPocket()
     */
    public byte getAvailablePockets() {
    	if(debugMode){System.out.println(java_debug_string+"GetAvailablePockets");}
        proxy = dll.getFunction(Utils.getMethods().get("GetAvailablePockets"));
        byte result[] = new byte[1];
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        } else {
            return 0;
        }
    }

    /**
     * This function permits to temporary downgrade a 2 pockets device to a 1 pocket device. It
        doesn’t update the IDCard. It’s intended for Demo purposes.<br>
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @param pocket Pocket number (1 or 2)
     * @return boolean
     * @see setFeedingMode()
     */
    public boolean setMaxPocket(byte pocket) {
    	if(debugMode){System.out.println(java_debug_string+"SetMaxPocket");}
        proxy = dll.getFunction(Utils.getMethods().get("SetMaxPocket"));
        Object[] par = new Object[]{deviceID, pocket};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     *  This function sets the feeding mode of the device. It is supported by VisionS only.
        There are two working modes:<br> - Start/Stop
        The device has the capability to wait for the pocket setting, even if the application
        takes a lot of time to decide the pocket. In case of long delays the document is stopped
        in the View station and it will be restarted after the SetPocket call.
        A document is fed only when the previous one has a destination pocket assigned. This
        means that the firmware feed a new doc when the application calls SetPocket to assign
        a destination to the previous one.
        - Flow-mode<br>
        The machine never stop the document in the view station and if the application takes
        too much time to decide the pocket, the device stops the job signalling document
        without destination or, if a default pocket is defined, sends the document to it ignoring
        eventual delayed SetPocket call.<br>
        A document is fed following the DPM throughput. This means that firmware feed<br>
        documents at a fixed throughput without waiting for the SetPocket call.
        Valid in State : DeviceChangeParameters<br>
        State Transition : none<br>
     * @param mode It’s the feeding mode. The options are defined in the header file.
     * @return boolean
     * @see setVirtualEndorsementProperty()
     */
    public boolean setFeedingMode(byte mode) {
    	if(debugMode){System.out.println(java_debug_string+"SetFeedingMode");}
        proxy = dll.getFunction(Utils.getMethods().get("SetFeedingMode"));
        Object[] par = new Object[]{deviceID, mode};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function gets the value of a property field of an endorsement area. The application is able
        to set all the area properties, even the predefined ones (Payee, BOFD and Transit).<br>
        Valid in State: DeviceChangeParameters, DeviceOnline<br>
        State Transition: none<br>
     * @param areaID It’s the area ID (see VApiInterface).
     * @param propID It’s the property ID (see VApiInterface).
     * @param pData It’s the user allocated buffer that contains the data.
     * @return boolean
     * @see getVirtualEndorsementProperty()
     */
    public boolean setVirtualEndorsementProperty(int areaID, int propID, int[] pData) {
    	System.out.println(java_debug_string+"SetVirtualEndorsementProperty");
        proxy = dll.getFunction(Utils.getMethods().get("SetVirtualEndorsementProperty"));
        Object[] par = new Object[]{deviceID, areaID, propID, pData};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

  /**
   *This function gets the value of a property field of an endorsement area.<br>
    Valid in State: DeviceChangeParameters, DeviceOnline<br>
    State Transition: none<br>
   * @param areaID It’s the area ID (see VApiInterface).
   * @param propID It’s the property ID
   * @return int[]
   * @see setVirtualEndorsementProperty()
   */
    public int[] getVirtualEndorsementProperty(int areaID, int propID) {
    	if(debugMode){System.out.println(java_debug_string+"GetVirtualEndorsementProperty");}
        proxy = dll.getFunction(Utils.getMethods().get("GetVirtualEndorsementProperty"));
        int[] result = new int[1];
        Object[] par = new Object[]{deviceID, areaID, propID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return result;
    }

    /**
     *  This function returns to the application the status of the feeder sensor to detect the presence of
        documents in the feeder. If the pFlag return value is TRUE, the feeder is empty. If FALSE the
        feeder contains one or more documents.<br>
        Valid in State : DeviceOnLine<br>
        State Transition : None<br>
     * @return boolean
     * @see startFeeding()
     */
    public boolean isFeederEmpty() {
    	if(debugMode){System.out.println(java_debug_string+"IsFeederEmpty");}
        proxy = dll.getFunction(Utils.getMethods().get("IsFeederEmpty"));
        boolean[] bFeederEmpty = {false};
        Object[] par = new Object[]{deviceID, bFeederEmpty};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return bFeederEmpty[0];
    }

    /**
     *  With this call the device begins to feed documents. The feeding mode is set in
        DeviceParameters structure (i.e. single or multiple document feeding, Main Hopper or Hand Drop mode).<br>
        Valid in State : DeviceOnLine<br>
        State Transition : DeviceFeeding<br>
     * @return boolean
     * @see stopFeeding()
     */
    public boolean startFeeding() {
    	if(debugMode){System.out.println(java_debug_string+"StartFeeding");}
        proxy = dll.getFunction(Utils.getMethods().get("StartFeeding"));
        Object[] par = new Object[]{deviceID};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     *  With this call the device stops feeding documents. If the Feeding mode is single document or
        if an exception occurs, StopFeeding call is not needed.
        Using a 30/60 DPM device, it works as a Start/Stop device while a 90 Dpm works as a flow
        mode one. Calling StopFeeding function before setting the destination pocket, the device will
        stop on the last fed document for a 30 or a 60 Dpm device, while for a 90 Dpm, due to its
        feeding feature, the feeding will stop after the next document. For example with a 30/60 DPM
        device it will be possible to stop the device, if a reject has been detected on a certain
        document, when that document will reach the destination pocket. That won’t be available on
        the 90 Dpm devices.<br>
        A 2 pocket machine, which is a Start/Stop device, is always capable to stop the machine on
        the last fed document.<br>
        Valid in State : DeviceFeeding<br>
        State Transition : DeviceOnLine<br>
     * @return boolean
     * @see startFeeding()
     */
    public boolean stopFeeding() {
    	if(debugMode){System.out.println(java_debug_string+"StopFeeding");}
        proxy = dll.getFunction(Utils.getMethods().get("StopFeeding"));
        Object[] par = new Object[]{deviceID};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * With this call the device enables the transport motor (at a lower speed) to free the track from
        jammed documents sending them to the desired Pocket. For a 2 pocket machine it’s possible
        to purge the track putting the documents in the second pocket. Using a 1 pocket machine the
        destination pocket must be 1.<br>
        Valid in State : DeviceOnLine<br>
        State Transition : none<br>
     * @param pocket the destination pocket in which the document will be purge
     * @return boolean
     * @see setPocket()
     */
    public boolean freeTrack(int pocket) {
    	if(debugMode){System.out.println(java_debug_string+"FreeTrack");}
        proxy = dll.getFunction(Utils.getMethods().get("FreeTrack"));
        Object[] par = new Object[]{deviceID, pocket};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * With this call the destination pocket for the current document is set, remember to call this
        function on WMPAR_SORTER_SET_ITEM_OUTPUT message. For a My Vision X with 1
        pocket the application must set always 1. For a 2 pocket machine valid values are 1 and 2.<br>
        Valid in State : DeviceFeeding (SetItemOutput event)<br>
        State Transition : none<br>
     * @param docNumber - the Identification number of the document
     * @param pocket - it’s the destination pocket in which the document will be sent
     * @return boolean
     * @see freeTrack()
     */
    public boolean setPocket(int docNumber, int pocket) {
    	if(debugMode){System.out.println(java_debug_string+"SetPocket");}
        proxy = dll.getFunction(Utils.getMethods().get("SetPocket"));
        Object[] par = new Object[]{deviceID, docNumber, pocket};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        return boolRet;
    }

    /**
     * This function returns the length of the last processed document in millimeters. Due to
        firmware limits this has to be considered as an informative value, and the expected error
        tolerance is +-3mm.<br>
        Valid in State : DeviceFeeding<br>
        State Transition : none<br>
     * @param docNumber the Identification number of the document
     * @param docLength int[] where the length of the document will be written
     * @return int
     * @see getMicrCodeline
     */
    public int getDocumentLength(int docNumber, int[] docLength) {
    	if(debugMode){System.out.println(java_debug_string+"GetDocumentLength");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDocumentLength"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, docNumber, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }
        return 0;
    }

    /**
     * This function returns the last recognized MICR codeline. The character after the ‘\0’ NULL
        terminator represents the font found for the recognized codeline. It will be ‘E’ for E13B or
        ‘C’ for CMC7 (this feature is useful only if MICR auto recognition is selected); ‘V’ means
        void string (no codeline).<br>
        Valid in State : DeviceFeeding (MicrAvailable event)<br>
        State Transition : none<br>
     * @return String
     * @see getOCRCodeline
     */
    public String getMicrCodeline() {
    	if(debugMode){System.out.println(java_debug_string+"GetMicrCodeline");}
        proxy = dll.getFunction(Utils.getMethods().get("GetMicrCodeline"));
        byte result[] = new byte[80];
        int maxLen = 80;
        String strResult = "";
        Object[] par = new Object[]{deviceID, result, maxLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strResult = Utils.byteToString(result);
        }else{
            strResult = " ErrorCode  " + getApiError();
        }
        return strResult;
    }

    /**
     * This function returns the recognized OCR codeline and Barcode. It is called to decode a
        memory bitmap with the OCR engine (including Bitmap File Header and Bitmap Info Header).
        The OCR and Barcode available fonts are:<br>
        OCR1_OCRA 0x0001 : OCR-A Euro limited<br>
        OCR1_OCRB 0x0002 : OCR-B Euro limited<br>
        OCR1_OCRAB 0x0003 : OCR-A and OCR-B Euro limited<br>
        OCR1_OCRBUK 0x0004 : OCR-B extended for UK banking<br>
        OCR1_E13BO 0x0005 : E13B optical<br>
        OCR1_E13BOXOCRBUK 0x0006 : E13B + OCRB for UK, switched on 'X'<br>
        OCR1_OCRAALNUM 0x0007 : OCR A alphanumeric<br>
        OCR1_OCRBALNUM 0x0008 : OCR B alphanumeric<br>
        OCR1_OCRB1403 0x0009 : OCRB 1403M<br>
        OCR1_OCRAN 0x000A : OCRA numeric for use with OCRB1403<br>
        OCR1_BC128 0x0010 : Barcode Code 128<br>
        OCR1_BC39 0x0011 : Barcode Code 39<br>
        OCR1_BC2OF5 0x0012 : Barcode Interleaved 2 of 5<br>
        OCR1_BCUPCA 0x0013 : Barcode UPCA<br>
        OCR1_BCEAN13 0x0014 : Barcode EAN 13<br>
        OCR1_BCUPCE 0x0015 : Barcode UPCE<br>
        OCR1_BCEAN8 0x0016 : Barcode EAN 8<br>
        OCR1_BCPDF417 0x0017 : Barcode PDF417<br>
        OCR1_CMC7O 0x0018 : CMC7 Optical<br>
        The valid range for the Reject threshold is 10-100, lower value generates more rejects, and
        reasonable values are between 35 and 55.<br>
        Valid in State : all<br>
        State Transition : none<br>
     * @param pBmp - byte[] to the image buffer
     * @param font - int OCR font
     * @param threshold - int Reject Threshold
     * @return String
     * @see  getMicrCodeline()
     */
    public String getOCRCodeline(byte[] pBmp, int font, int threshold) {
    	if(debugMode){System.out.println(java_debug_string+"GetOCRCodeline");}
        proxy = dll.getFunction(Utils.getMethods().get("GetOCRCodeline"));
        byte result[] = new byte[250];
        int maxLen = 250;
        String strResult = "";
        Object[] par = new Object[]{deviceID, pBmp, result, maxLen, font, threshold};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strResult = Utils.byteToString(result);
        }
        return strResult;
    }

    /**
     *  This function is used to define the printer text and bitmap data, their position on the document
        and the font used to create the text.<br>
        The bitmap image can be loaded from a file, or passed directly by its memory address (as a
        DIB, Device Independent Bitmap).<br>
        All Windows fonts can be selected and used for the printed text. Printer default font has been
        chosen to optimize uppercase characters usage. Lowercase letters could lose some detail at the
        very upper or very lower end, and in this case a font size reduction could be needed.
        Valid in State: Smart Printer disabled<br>
        This function has to be called in DeviceOnline state, before the
        StartFeeding call, for the first document. For the next documents, in
        DeviceFeeding state, has to be called during<br>
        WMPAR_SORTER_NEW_DOCUMENT. For 30 and 60 DPM machine<br>
        this function can be called during<br>
        WMPAR_SORTER_SET_ITEM_OUTPUT, instead of<br>
        WMPAR_SORTER_NEW_DOCUMENT.<br>
        When the Smart Printer is disabled, the printer is an up-stream device. This
        means that the printer information have to be defined before the document
        feeding. Thus, the call before StartFeeding defines the printer data for the
        first document. The following calls, during<br>
        WMPAR_SORTER_NEW_DOCUMENT message, define the printer data<br>
        for the next document.<br>
        Example of printer sequence:<br>
        1. SendPrinterData(…) for the 1st doc<br>
        2. StartFeeding(…)<br>
        3. During NEW_DOC message of the 1st doc call SendPrinterData(…)for the 2nd doc<br>
        4. During NEW_DOC message of the nth doc call<br>
        SendPrinterData(…) for the n+1th doc<br>
        Smart Enabled<br>
        This function has to be called during the
        WMPAR_SORTER_SET_ITEM_OUTPUT message for all the machines.<br>
        When Smart printer is enabled, the printer is a downstream device. This
        means that the printer information can be defined after MICR and/or OCR<br>
        information are available.<br>
        Example of printer sequence:<br>
        1. Call StartFeeding(…)<br>
        2. During SET_ITEM_OUTPUT message of the 1st doc call SendPrinterData(…) for the 1st doc<br>
        3. During SET_ITEM_OUTPUT message of the nth doc call<br>
        SendPrinterData(…) for the nth doc<br>
        State Transition: none<br>
     * @param dwHead  Printing head selector. Must be always 0.
     * @param logFont  Font descriptor. Setting this structure to all zero means default font.
                        My Vision X AGP default is Arial, 16, normal.
                        My Vision X no AGP (single line) default is Arial,12, normal.
     * @param sText User-allocated string containing the text to be printer. Capital letters are suggested.
                        A NULL value means that no text will be printed.
     * @param dwTextOffset Horizontal position of the printing text on the document referred to the leading or to the trailing edge. The position is expressed in mm.
     * @param sImgPath User-allocated string containing the path to the Bitmap image or user-allocated buffer pointing the DIB (Device Independent Bitmap) in memory.
                        A NULL value means that no image will be printed.
     * @param mgOffset Horizontal Position of the image on document referred to leading or the trailing edge. . The position is expressed in mm.
                         Since API version 2.11.1.2 this value can express a vertical position in pixels. This option is available
                         only for the AGP with 2 lines enabled and when there’s no text to print (sText is NULL or empty).
                         The position is expressed in pixels. The values range is form 0 to 49. The LSBytes of the DWORD must contain the horizontal position in mm. The MSBytes must contain the vertical position in pixels.
     * @param dwImgSrcType Define sImgPath parameter type. Values are:
                             PRT_SRC_FILE: sImgPath is a path to a file
                             PRT_SRC_MEM_PTR: sImgPath is a pointer to a memory buffer
     * @return boolean
     * @see setVirtualEndorsement
     */
    public boolean sendPrinterData(int dwHead,LogFont logFont, byte[] sText, int dwTextOffset, byte[] sImgPath, int mgOffset, int dwImgSrcType) {
        if(debugMode){System.out.println(java_debug_string+"SendPrinterDataHelp");}
        proxy = dllHelper.getFunction(Utils.getMethods().get("SendPrinterDataHelp"));
        //int SendPrintDataHelp(unsigned long,unsigned long,struct tagLOGFONTA *,char *,unsigned long,char *,unsigned long,unsigned long)
        Object[] par = new Object[]{deviceID,dwHead,logFont, sText, dwTextOffset, sImgPath, mgOffset, dwImgSrcType};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /*
    public boolean sendPrinterDataHelp(int dwHead, LogFont logFont, byte[] sText, int dwTextOffset, byte[] sImgPath, int mgOffset, int dwImgSrcType) {
    	if(debugMode){System.out.println(java_debug_string+"SendPrinterData");}
        proxy = dll.getFunction(Utils.getMethods().get("SendPrinterData"));
        Object[] par = new Object[]{deviceID, dwHead, logFont, sText, dwTextOffset, sImgPath, mgOffset, dwImgSrcType};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }
    */
    
    /**
     * This function is used to define the printer text and the bitmap data of a certain Virtual
        endorsement area during the document processing.
        The text is a string that contains one or more lines separated by a ‘\n’ character.
        The bitmap image can be loaded from a file.<br>
        Valid in State: Smart Printer disabled<br>
        This function has to be called in DeviceOnline state, before the
        StartFeeding call, for the first document. For the next documents, in
        DeviceFeeding state, has to be called during
        WMPAR_SORTER_NEW_DOCUMENT. For 30 and 60 DPM machine<br>
        this function can be called during<br>
        WMPAR_SORTER_SET_ITEM_OUTPUT, instead of<br>
        WMPAR_SORTER_NEW_DOCUMENT.<br>
        When the Smart Printer is disabled, the printer is an up-stream device. This
        means that the printer information have to be defined before the document
        feeding. Thus, the call before StartFeeding defines the printer data for the
        first document. The following calls, during<br>
        WMPAR_SORTER_NEW_DOCUMENT message, define the printer data<br>
        for the next document.<br>
        Example of printer sequence:<br>
        5. SetVirtualEndorsement(…) for the 1st doc<br>
        6. StartFeeding(…)<br>
        7. During NEW_DOC message of the 1st doc call<br>
        SetVirtualEndorsement (…) for the 2nd doc<br>
        8. During NEW_DOC message of the nth doc call<br>
        SetVirtualEndorsement (…) for the n+1th doc<br>
        Smart Enabled<br>
        This function has to be called during the<br>
        WMPAR_SORTER_SET_ITEM_OUTPUT message for all the machines.<br>
        When Smart printer is enabled, the printer is a downstream device. This
        means that the printer information can be defined after MICR and/or OCR
        information are available.<br>
        Example of printer sequence:<br>
        4. Call StartFeeding(…)<br>
        5. During SET_ITEM_OUTPUT message of the 1st doc call<br>
        SetVirtualEndorsement (…) for the 1st doc<br>
        6. During SET_ITEM_OUTPUT message of the nth doc call<br>
        SetVirtualEndorsement (…) for the nth doc<br>
        Valid in State: DeviceOnline, DeviceFeeding (SetItemOutput event)<br>
        State Transition: none<br>
     * @param AreaID Sets the Area ID. Its values are defined in VApiInterface
     * @param sText The text to be printed on the image. It can contain multiple lines of text separated by a ‘\n’ character.
     * @param sBmpPath The path to a bitmap file. It can be a 1, 8 or 24 bpp image. The bitmap are automatically converted in the
                         color space of the image (i.e. color converted in greyscale, color in black and white and greyscale in black and white). If the bitmap header contains the
                         resolution of the image, it is automatically rescaled to the document image resolution. If the bitmap header
                         doesn’t define any resolution it is copied pixelwise on the document image
     * @param bmpSrcType Sets the bitmap type. Only file path are admitted. Its value is defined in VApiInterface
     * @return boolean
     * @see sendPrinterData()
     */
    public boolean setVirtualEndorsement(int AreaID, byte[] sText, byte[] sBmpPath, int bmpSrcType) {
    	if(debugMode){System.out.println(java_debug_string+"setVirtualEndorsement");}
        proxy = dll.getFunction(Utils.getMethods().get("SetVirtualEndorsement"));
        Object[] par = new Object[]{deviceID, AreaID, sText, sBmpPath, bmpSrcType};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     *  This function Frees the memory used for the Image buffers, the application must call this
            method when the buffers are no longer needed.<br>
            Valid in State : all<br>
            State Transition : none<br>
     * @param imagesStruct - the Image Structure to be released.
     * @return boolean
     * @see freeSnippetBuffer()
     */
    public boolean freeImagesBuffer(Pointer imagesStruct) {
    	if(debugMode){System.out.println(java_debug_string+"FreeImagesBuffer");}
        proxy = dll.getFunction(Utils.getMethods().get("FreeImagesBuffer"));
        Object[] par = new Object[]{imagesStruct};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     * This function Frees the memory used for the Snippet buffers, the application must call this
        method when the buffers are no longer needed.<br>
        Valid in State : all<br>
        State Transition : none<br>
     * @param imagesStruct the Snippet Structure to be released
     * @return boolean
     * @see freeImagesBuffer()
     */
    public boolean freeSnippetBuffer(Pointer imagesStruct) {
    	if(debugMode){System.out.println(java_debug_string+"FreeSnippetBuffer");}
        proxy = dll.getFunction(Utils.getMethods().get("FreeSnippetBuffer"));
        Object[] par = new Object[]{imagesStruct};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }


    /**
     *  This function disables on the fly the Rear image capture.
        If the Rear image is not enabled the application hasn’t to call it. If the Rear image requested
        the app can decide whether to call it or not.<br>
        If not called and the Rear is enabled, the image is captured and compressed and the
        application will receive a valid pointer in the ImagesStruct.<br>
        If called, the Rear capture is cancelled and the application will receive a NULL pointer in the
        ImagesStruct. The function must be called before the SetPocket.<br>
        It is available only for VisionS device.<br>
        Valid in State: DeviceFeeding<br>
        State Transition: none<br>
     * @param DocID the document ID.
     * @return boolean
     * @see getDoubleFeedingResult
     */
    public boolean disableDownStreamImage(int DocID) {
    	if(debugMode){System.out.println(java_debug_string+"DisableDownStreamImage");}
        proxy = dll.getFunction(Utils.getMethods().get("DisableDownStreamImage"));
        Object[] par = new Object[]{deviceID, DocID};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     *  This function returns the result of the double-feed detection. When a multiple feed has been
        detected pbDetected is set to TRUE. If the Double-feeding detection is disabled the function
        always return FALSE in the pbDetected parameter.<br>
        It is available only for VisionS device. VisionX doesn’t support it.<br>
        Valid in State: DeviceFeeding<br>
        State Transition: none<br>
     * @param dwDocID the document ID.
     * @param pbDetected receive the double-feeding result. TRUE means detected.
     * @return boolean
     * @see disableDownStreamImage()
     */
    public boolean getDoubleFeedingResult(int dwDocID, boolean[] pbDetected) {
    	if(debugMode){System.out.println(java_debug_string+"GetDoubleFeedingResult");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDoubleFeedingResult"));
        Object[] par = new Object[]{deviceID, dwDocID, pbDetected};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     *  This function returns the status of the Full Pockets detection.
            This function is supported by the VisionS only.<br>
            There are two kind of full pocket messages:<br>
            1. Full pocket warning<br>
            It means that a pocket has become almost full. The VisionS device doesn’t stop the
            process. Calling the GetFullPocketStatus the application get the actual status and it
            can exclude the almost full pocket from the sorting, allowing the operator to remove
            from the pocket. All this operations can be executed without stopping the process.
            If after this warning the app continue to put documents in the “almost full” pockets the
            stop the job after N documents signalling a stop for Full Pocket condition.
            N is a firmware parameter.<br>
            2. Full pocket<br>
            It means that a pocket is completely full. The VisionS device automatically stops the
            process completing the pending documents and the device state returns to OnLine.
            The parameter pStatus must be an array of 2 DWORD.The LSb of the 1st DWORD is the
            pocket #1. MSb is the pocket #32. The LSb of the 2nb DWORD is the pocket #33. The Msb is
            the pocket #64. Each bit represent the Full Pocket status. A bit set to 1 means Full Pocket detected.<br>
            Valid in State: DeviceOnline, DeviceFeeding<br>
            State Transition: none<br>
     * @param pStatus It’s a int[] to the destination buffer. It must be an array of two DWORDs.
     * @param bWarning If it is set to TRUE the function returns the Full pocket
              warning status. If it is set to FALSE the function return the completely Full pocket status.
     * @return boolean
     * @see getDocumentType()
     */
    public boolean getFullPocketStatus(int[] pStatus, boolean bWarning) {
    	if(debugMode){System.out.println(java_debug_string+"GetFullPocketStatus");}
        proxy = dll.getFunction(Utils.getMethods().get("GetFullPocketStatus"));
        Object[] par = new Object[]{deviceID, pStatus, bWarning};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     * This function has to be called during one of the following messates:<br>
            - NEW_DOC<br>
            - MICR_AVAILABLE<br>
            - SET_ITEM_OUPUT<br>
            It returns, in the pointer DocType, a value that represents if the item is a check or a plastic card.
            The available types are:<br>
            - DOC_TYPE_CHEQUE<br>
            - DOC_TYPE_ID_CARD.<br>
            Valid in State : DeviceFeeding<br>
            State Transition : none<br>
     * @param DocID the Identification of the Document
     * @return int type of processed document.
     * @see getFullPocketStatus()
     */
    public int getDocumentType(int DocID) {
    	if(debugMode){System.out.println(java_debug_string+"GetDocumentType");}
        proxy = dll.getFunction(Utils.getMethods().get("GetDocumentType"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, DocID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }else{
            return 0;
        }
    }

    /**
     * This function is used to enable the franking on the check, it needs to be called during the
            SET_DOWNSTREAM_OPTIONS message.<br>
            Valid in state: Feeding.<br>
            State Transition: none<br>
     * @param trailingEdgeAlignment It defines the alignment of the printed text (LEADING or TRAILING).
     * @param startPosition It’s the starting position in mm from the alignment edge of the document.
     * @return boolean
     * @see isTrackFree()
     */
    public boolean enableFranking(int trailingEdgeAlignment, int startPosition) {
    	if(debugMode){System.out.println(java_debug_string+"EnableFranking");}
        proxy = dll.getFunction(Utils.getMethods().get("EnableFranking"));
        Object[] par = new Object[]{deviceID, trailingEdgeAlignment, startPosition};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     * This function returns the status of the track. If all the sensors are free, it means that the track
            is free and a new document/card can be processed. If pFlag is set to TRUE the track is free.
            Otherwise one or more sensors are busy due to a document/card stopped in the track<br>
            Valid in State : DeviceOnline<br>
            State Transition : none<br>
     * @return boolean
     * @see enableFranking()
     */
    public boolean isTrackFree() {
    	if(debugMode){System.out.println(java_debug_string+"IsTrackFree");}
        proxy = dll.getFunction(Utils.getMethods().get("IsTrackFree"));
        boolean result[] = new boolean[1];
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }else{
            return false;
        }
    }

    /**
     * Set the serial port BaudRate. Valid values are from 300 to 57600.<br>
            Valid in State : DeviceChangeParameters<br>
            State Transition : none<br>
     * @param baudRate
     * @return boolean
     * @see rs232Write()
     */
    public boolean rs232SetBaud(int baudRate) {
    	if(debugMode){System.out.println(java_debug_string+"Rs232SetBaud");}
        proxy = dll.getFunction(Utils.getMethods().get("Rs232SetBaud"));
        Object[] par = new Object[]{deviceID, baudRate};
        return  (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     * This function sends pBuffer data through the serial port.<br>
            Valid in State : DeviceOnLine, DeviceFeeding<br>
            State Transition : none<br>
     * @param pBuffer
     * @return boolean
     * @see  rs232SetBaud()
     */
    public boolean rs232Write(byte[] pBuffer) {
    	if(debugMode){System.out.println(java_debug_string+"Rs232Write");}
        proxy = dll.getFunction(Utils.getMethods().get("Rs232Write"));
        Object[] par = new Object[]{deviceID, pBuffer, pBuffer.length};
        return  (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     *  This function reads the length of the data contained in the device reception buffer.<br>
            This length should be used to call Rs232Read.<br>
            Valid in State : DeviceOnLine, DeviceFeeding<br>
            State Transition : none<br>
     * @return int
     * @see rs232Read()
     */
    public int rs232GetLen() {
    	if(debugMode){System.out.println(java_debug_string+"Rs232GetLen");}
        proxy = dll.getFunction(Utils.getMethods().get("Rs232GetLen"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }else{
            return 0;
        }
    }

   /**
    * This function transfers the data contained in the device reception buffer. The length of the
        data to be transferred is indicated by a previous call to Rs232GetLen.<br>
        Valid in State : DeviceOnLine, DeviceFeeding<br>
        State Transition : none<br>
    * @param len The length of the data to be transferred in pBuffer. Valid values
                 are from 1 to 255 and must be less than or equal to the value returned by Rs232GetLen.
    * @return String
    * @see rs232GetLen()
    */
    public String rs232Read(int len) {
    	if(debugMode){System.out.println(java_debug_string+"Rs232Read");}
        proxy = dll.getFunction(Utils.getMethods().get("Rs232Read"));
        byte result[] = new byte[len];
        String strResult = "";
        Object[] par = new Object[]{deviceID, result, len};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strResult = Utils.byteToString(result);
        }
        return strResult;
    }

    /**
     * The device is able to count the number of ink drops fired by the printer cartridge enabling the
            user application to monitor the cartridge consumption status. This function reads the printer
            cartridge’s drops counter. It’s available for every kind of printer device (Single line and AGP multi-line).
            Valid in State : DeviceOnLine
            State Transition : none
     * @return int
     * @see ResetPrinterDropsCounter()
     */
    public int readPrinterDropsCounter() {
    	if(debugMode){System.out.println(java_debug_string+"ReadPrinterDropsCounter");}
        proxy = dll.getFunction(Utils.getMethods().get("ReadPrinterDropsCounter"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }else{
            return 0;
        }
    }

    /**
     *  This function resets the drops counter for the printer cartridge. This function should be used to
            reset the drops counter after the ink-jet cartridge replacement. The user application is
            responsible for deciding when the ink cartridge needs to be replaced, refer to the relative
            technical bulletin for the respective ink quantity in each type of ink cartridge.
            Valid in State : DeviceOnLine
            State Transition : none
     * @return boolean
     * @see readPrinterDropsCounter()
     */
    public boolean ResetPrinterDropsCounter() {
    	if(debugMode){System.out.println(java_debug_string+"ResetPrinterDropsCounter");}
        proxy = dll.getFunction(Utils.getMethods().get("ResetPrinterDropsCounter"));
        Object[] par = new Object[]{deviceID};
        return  (Boolean) proxy.invoke(Boolean.class, par);
    }

   /**
    *  This function has the capability to detect the presence of the ink-jet cartridge in the device
        nest (Single line and AGP both) and can return the cartridge ID number for the AGP version.
        This ID can be used to detect when a cartridge is replaced. The presence flag can be used,
        when the printer is enabled, to check if the printer cartridge is correctly mounted before
        starting a printing job.<br>
        It can be called before starting a printing job to verify if the cartridge is correctly mounted.
        This feature is not available for the previous version of the MyVisionX. For the previous
        version of the device the cartridge is considered always inserted and the ID is always 0.
        Valid in State : DeviceOnLine
        State Transition : none
    * @param bInserted receive the presence flag. If TRUE means cartridge inserted. If FALSE means cartridge not mounted.
    * @return int
    * @see getMagCardResult
    */
    public int getPrinterCartridgeInfo(boolean[] bInserted) {
    	if(debugMode){System.out.println(java_debug_string+"GetPrinterCartridgeInfo");}
        proxy = dll.getFunction(Utils.getMethods().get("GetPrinterCartridgeInfo"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, bInserted, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }else{
            return 0;
        }
    }

    /**
     *  This function returns the values of some historical counters.
        The available counters are:<br>
        - COUNTER_DOCS<br>
        Return how many documents have been processed during the product life.<br>
        - COUNTER_CARDS<br>
        Return how many plastic cards have been processed during the product life.<br>
        - COUNTER_FRANKING<br>
        Return how many documents have been franked. This can be used in order to detect
        when the franking roller has to be replaced.
        Valid in State : DeviceOnline
        State Transition : none
     * @param counterID It selects the counter
     * @return int
     * @see resetHistoricalCounter
     */
    public int getHistoricalCounter(int counterID) {
    	if(debugMode){System.out.println(java_debug_string+"GetHistoricalCounter");}
        proxy = dll.getFunction(Utils.getMethods().get("GetHistoricalCounter"));
        int result[] = new int[1];
        Object[] par = new Object[]{deviceID, counterID, result};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            return result[0];
        }else{
            return 0;
        }
    }

    /**
     * This function reset some specific historical counters of consumable parts.
        The available counters are:<br>
        - COUNTER_FRANKING<br>
        Return how many documents have been franked. This can be used in order to reset the
        counter when the franking roller has to be replaced.
        Valid in State : DeviceOnline
        State Transition : none
     * @param counterID It selects the counter
     * @return boolean
     * @see getHistoricalCounter()
     */
    public boolean resetHistoricalCounter(int counterID) {
    	if(debugMode){System.out.println(java_debug_string+"ResetHistoricalCounter");}
        proxy = dll.getFunction(Utils.getMethods().get("ResetHistoricalCounter"));
        Object[] par = new Object[]{deviceID, counterID};
        return  (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     * This function has to be called when the application receive the message
            WMPAR_MAGCARD. When the magcard has been decoded without error, it returns an
            ASCII string. When the standard of the magcard bit-stream it’s not recognized, the function
            return the raw bit-stream data in order to permit to the application to decode the magcard by
            itself. When parity or direction error has been detected no data will be returned.
            This function is not available for the previous version of the MyVisionX.
            Valid in State: DeviceOnLine
            State Transition: none
     * @param bufferLen it’s the sResult buffer length. We suggest to allocate at least MAGCARD_MIN_BUFFER_LEN bytes.
     * @return String
     * @see getPrinterCartridgeInfo()
     */
    public String getMagCardResult(int bufferLen) {
    	if(debugMode){System.out.println(java_debug_string+"GetMagCardResult");}
        proxy = dll.getFunction(Utils.getMethods().get("GetMagCardResult"));
        byte result[] = new byte[bufferLen];
        String strResult = "";
        Object[] par = new Object[]{deviceID, result, bufferLen};
        boolRet = (Boolean) proxy.invoke(Boolean.class, par);
        if (boolRet) {
            strResult = Utils.byteToString(result);
        }
        return strResult;
    }

    /**
     * This function Frees the memory used for the Image buffers, the application must call this
            method when the buffers are no longer needed.<br>
            Valid in State : all<br>
            State Transition : none<br>
     * @param imagesStruct
     * @return boolean
     * @see freeSnippetBuffer
     */
    public boolean freeImagesBuffer(ImagesStruct[] imagesStruct) {
    	if(debugMode){System.out.println(java_debug_string+"FreeImagesBuffer");}
        proxy = dll.getFunction(Utils.getMethods().get("FreeImagesBuffer"));
        Object[] par = new Object[]{imagesStruct};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    /**
     * This function Frees the memory used for the Snippet buffers, the application must call this
            method when the buffers are no longer needed.<br>
            Valid in State : all<br>
            State Transition : none<br>
     * @param imagesStruct
     * @return boolean
     * @see freeImagesBuffer()
     */
    public boolean freeSnippetBuffer(ImagesStruct[] imagesStruct) {
    	if(debugMode){System.out.println(java_debug_string+"FreeSnippetBuffer");}
        proxy = dll.getFunction(Utils.getMethods().get("FreeSnippetBuffer"));
        Object[] par = new Object[]{imagesStruct};
        return (Boolean) proxy.invoke(Boolean.class, par);
    }

    public void destroy() {
        dll.dispose();
    }

}
